import {PlusOutlined} from '@ant-design/icons';
import {Button, Divider, message, Modal, Radio} from 'antd';
import React, {useRef, useState} from 'react';
import {PageHeaderWrapper} from '@ant-design/pro-layout';
import ProTable, {ActionType, ProColumns} from '@ant-design/pro-table';
import {SorterResult} from 'antd/es/table/interface';
import QueryForm from '@/pages/ModelList/components/QueryForm';
import {TableListItem} from './data.d';
import service from './service';

/**
 * 添加节点
 * @param fields
 */
const handleAdd = async (fields: TableListItem) => {
  const hide = message.loading('正在添加');
  try {
    const res = await service.addModel({...fields});
    hide();
    if (res.success) {
      message.success('添加成功');
      return true;
    }
    message.error('添加失败请重试！');
    return false;

  } catch (error) {
    hide();
    message.error('添加失败请重试！');
    return false;
  }
};

/**
 * 更新节点
 * @param fields
 */
const handleUpdate = async (fields: TableListItem) => {
  const hide = message.loading('正在配置');
  try {
    const res = await service.updateModel(fields);
    hide();
    if (res.success) {
      message.success('配置成功');
      return true;
    }
    message.error('配置失败请重试！');
    return false;
  } catch (error) {
    hide();
    message.error('配置失败请重试！');
    return false;
  }
};

/**
 *  删除节点
 * @param fields
 */
const handleRemove = async (fields: TableListItem) => {
  const hide = message.loading('正在删除');
  try {
    const res = await service.removeModel(fields);
    hide();
    message.success('删除成功，即将刷新');
    return res.success;
  } catch (error) {
    hide();
    message.error('删除失败，请重试');
    return false;
  }
};

const TableList: React.FC<{}> = () => {
  const [sorter, setSorter] = useState<string>('');
  const [modelVisible, setModalVisible] = useState<boolean>(false);
  const [formValues, setFormValues] = useState({});
  const actionRef = useRef<ActionType>();
  const [queryVisible, setQueryVisible] = useState<boolean>(false);

  const columns: ProColumns<TableListItem>[] = [
    {
      title: '模型类型',
      dataIndex: 'modelType',
      hideInSearch: true,
      width: 120,
      rules: [
        {
          required: true,
          message: '模型类型为必填项',
        },
      ],
      renderFormItem: () => {
        return (
          <Radio.Group>
            <Radio value={0}>albert</Radio>
            <Radio value={1}>bilstm</Radio>
          </Radio.Group>
        );
      },
      valueEnum: {
        0: {text: 'albert'},
        1: {text: 'bilstm'},
      },
    },
    {
      title: '模型名称',
      dataIndex: 'modelName',
      width: 150,
      copyable: true,
      rules: [
        {
          required: true,
          message: '模型名称为必填项',
        },
      ],
    },
    {
      title: '模型文件',
      dataIndex: 'modelFile',
      hideInSearch: true,
      width: 200,
      copyable: true,
      rules: [
        {
          required: true,
          message: '模型文件为必填项',
        },
      ],
    },
    {
      title: '向量文件',
      dataIndex: 'vocabFile',
      hideInSearch: true,
      width: 200,
      copyable: true,
      rules: [
        {
          required: true,
          message: '词向量文件为必填项',
        },
      ],
    },
    {
      title: '标签文件',
      dataIndex: 'labelFile',
      hideInSearch: true,
      width: 200,
      copyable: true,
      rules: [
        {
          required: true,
          message: '标签文件为必填项',
        },
      ],
    },
    {
      title: '必包含',
      dataIndex: 'mustTag',
      hideInSearch: true,
      copyable: true,
      formItemProps: {
        placeholder: '逗号分隔'
      },
      hideInTable: true,
    },
    {
      title: '必不包含',
      dataIndex: 'mustNotTag',
      hideInSearch: true,
      copyable: true,
      formItemProps: {
        placeholder: '逗号分隔'
      },
      hideInTable: true,
    },

    {
      title: 'BatchSize',
      dataIndex: 'maxBatchSize',
      hideInSearch: true,
      hideInTable: true,
    },
    {
      title: 'SeqLength',
      dataIndex: 'maxSeqLength',
      hideInSearch: true,
      hideInTable: true,
    },

    {
      title: 'HiddenLayers',
      dataIndex: 'numHiddenLayers',
      hideInSearch: true,
      hideInTable: true,
    },
    {
      title: 'AttentionHeads',
      dataIndex: 'numAttentionHeads',
      hideInSearch: true,
      hideInTable: true,
    },
    {
      title: '纯文本',
      dataIndex: 'onlyWord',
      hideInSearch: true,
      hideInTable: true,
      renderFormItem: () => {
        return (
          <Radio.Group>
            <Radio value="true">是</Radio>
            <Radio value="false">否</Radio>
          </Radio.Group>
        );
      },
      valueEnum: {
        true: {text: '是'},
        false: {text: '否'},
      },
    },
    {
      title: '大小写',
      dataIndex: 'doLowerCase',
      hideInSearch: true,
      hideInTable: true,
      renderFormItem: () => {
        return (
          <Radio.Group>
            <Radio value={0}>区分</Radio>
            <Radio value={1}>不分区</Radio>
          </Radio.Group>
        );
      },
      valueEnum: {
        0: {text: '区分'},
        1: {text: '不分区'},
      },
    },

    {
      title: '操作',
      width: 200,
      dataIndex: 'option',
      valueType: 'option',
      render: (_, record,index,action) => (
        <>
          <a
            onClick={() => {
              setFormValues(record);
              setModalVisible(true);
              console.log(record);
            }}
          >
            配置
          </a>
          <Divider type="vertical"/>
          <a
            onClick={async () => {
              const success = await  handleRemove(record);
              if (success) {
                await action.reload();
              }
              console.log(record);
            }}
          >
            删除
          </a>
          <Divider type="vertical"/>
          <a
            onClick={async () => {
              setFormValues(record);
              setQueryVisible(true);
            }}
          >
            查询
          </a>
        </>
      ),
    },
  ];

  return (
    <PageHeaderWrapper>
      <ProTable<TableListItem>
        headerTitle="查询表格"
        actionRef={actionRef}
        rowKey="id"
        onChange={(_, _filter, _sorter) => {
          const sorterResult = _sorter as SorterResult<TableListItem>;
          if (sorterResult.field) {
            setSorter(`${sorterResult.field}_${sorterResult.order}`);
          }
        }}
        pagination={{
          defaultPageSize: 10,
        }}
        params={{
          sorter,
        }}
        /* eslint-disable */
        toolBarRender={(action, {selectedRows}) => [
          <Button
            type="primary"
            onClick={() => {
              setModalVisible(true);
              setFormValues({
                maxBatchSize: 1,
                maxSeqLength: 40,
                numAttentionHeads: 12,
                numHiddenLayers: 3,
                doLowerCase: "1",
                onlyWord: "true",
              });
            }}
          >
            <PlusOutlined/> 新建
          </Button>,
        ]}
        request={(params) => service.queryModel(params)}
        columns={columns}
        rowSelection={{}}
      />
      <Modal
        destroyOnClose
        title={'id' in formValues ? '编辑' : '新建'}
        visible={modelVisible}
        onCancel={() => setModalVisible(false)}
        footer={null}
      >
        <ProTable<TableListItem, TableListItem>
          onSubmit={async (value) => {
            let success = false;
            if ('id' in formValues) {
              value['id'] = formValues['id']
              success = await handleUpdate(value);
            } else {
              success = await handleAdd(value);
            }
            if (success) {
              setModalVisible(false);
              if (actionRef.current) {
                actionRef.current.reload();
              }
            }
          }}
          rowKey="id"
          type="form"
          columns={columns}
          rowSelection={{}}
          form={{initialValues: formValues}}
        />
      </Modal>

      {queryVisible ? (
        <QueryForm
          onCancel={() => setQueryVisible(false)}
          modalVisible={queryVisible}
          values={formValues}
        />
      ) : null}

    </PageHeaderWrapper>
  );
};

export default TableList;
